﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls;
using System.Collections;

namespace TreeViewToTreeViewExample
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();

			ObservableCollection<IDataItem> items = new ObservableCollection<IDataItem>();

			Category category1 = new Category("Category1");
			category1.Services.Add(new Service("Service 1"));
			category1.Services.Add(new Service("Service 2"));

			Category category2 = new Category("Category2");
			category2.Services.Add(new Service("Service 3"));
			category2.Services.Add(new Service("Service 4"));

			items.Add(category1);
			items.Add(category2);

			leftTreeView.ItemsSource = items;

			RadDragAndDropManager.SetAllowDrag(leftTreeView, true);

			RadDragAndDropManager.AddDragQueryHandler(leftTreeView, LeftTreeView_OnDragQuery);
			rightTreeView.AddHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(RightTreeView_OnDropQuery), true);
		}

		private void RightTreeView_OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			var rightTreeViewServices = rightTreeView.Items;
			var draggedItems = e.Options.Payload as ICollection;
			List<Service> draggedServices = new List<Service>();

			foreach (var item in draggedItems)
			{
				if(item is Service)
				{
					draggedServices.Add(item as Service);
				}
			}

			if(draggedServices.Count == 0)
			{
				e.QueryResult = false;
			}

			if (e.Options.Source is RadTreeView && e.Options.Source != sender)
			{
				foreach (var rightTreeViewService in rightTreeViewServices)
				{
					foreach (var draggedService in draggedServices)
					{
						if (rightTreeViewService == draggedService)
						{
							e.QueryResult = false;
						}
					}
				}
			}

			if ((e.Options.Destination as RadTreeViewItem) != null && (e.Options.Destination as RadTreeViewItem).DropPosition == DropPosition.Inside)
			{
				e.QueryResult = false;
			}
		}

		private void LeftTreeView_OnDragQuery(object sender, DragDropQueryEventArgs e)
		{
			RadTreeView leftTreeView = sender as RadTreeView;

			e.Options.Payload = leftTreeView.SelectedItems.ToList();
			e.Options.DragCue = new ItemsControl()
			{
				ItemsSource = leftTreeView.SelectedItems,
				DisplayMemberPath = "Name"
			};
			e.QueryResult = true;
			e.Handled = true;
		}
	}

	public class Service : IDataItem
	{
		public Service(string name)
		{
			this.Name = name;
		}
		public string Name { get; set; }
	}

	public class Category : IDataItem
	{
		public Category(string name)
		{
			this.Name = name;
			this.Services = new ObservableCollection<Service>();
		}

		public string Name { get; set; }
		public ObservableCollection<Service> Services { get; set; }
	}

	interface IDataItem
	{
		string Name { get; set; }
	}
}
